/**
 * Create a dialog to generate a slider for choosing uninstalled apps.
 *
 * The .contextualappadsUiAppsdialog() creates a dialog which enables the user to slide between relevant apps which could be useful for his context. Hovering over a recommended app shows more information of the app.
 *
 * ### Examples
 * Binds the contextualappadsUiAppsdialog()-Dialog to the following div and calls the open-method of the widget via on-click event attached to the page element.
 *
 * HTML:
 * <div class="ep-top-bar-buttons ep-top-bar-buttons-context-apps"></div>
 *
 * JavaScript:
 *
 *    <script>
 *        require([
 *           'jquery',
 *           'ep',
 *           'de_epages/contextualappads/ui/appsdialog',
 *
 *           '$ready!'
 *       ], function($, ep, de_epages) {
 *           var appsForContext = [{
 *                   appUrl: 'https://www.billbee.de',
 *                   description: 'Billbee bietet eine umfangreiche aber einfach zu bedienende...',
 *                   id: '1',
 *                   name: 'Billbee',
 *                   installed: false,
 *                   logoUrl: 'bild1.jpg'
 *               }, {
 *                   appUrl: 'https://www.facebook.de',
 *                   description: 'Facebook bietet eine umfangreiche aber einfach zu bedienende...',
 *                   id: '2',
 *                   name: 'Facebook',
 *                   installed: false,
 *                   logoUrl: 'bild2.jpg'
 *               }, {
 *                   appUrl: 'https://www.whatsapp.de',
 *                   description: 'Whatsapp bietet eine umfangreiche aber einfach zu bedienende...',
 *                   id: '3',
 *                   name: 'WhatsApp',
 *                   installed: true,
 *                   logoUrl: 'bild3.jpg'
 *               }];
 *
 *           $('.ep-top-bar-buttons-context-apps').contextualappadsUiAppsdialog({
 *               apps: appsForContext,
 *               shopId: #Shop.ID
 *           });
 *       });
 *   </script>
 *
 *
 *
 * @class jQuery.ui.contextualappadsUiAppsdialog
 * @extends jQuery.widget
 *
 * @uses ep
 * @uses de_epages
 * @uses $tmpl!./appsdialog
 * @uses jquery/slick/slick
 * @uses ep.dict
 * @uses jQuery.ui.widget
 * @uses ep.ui.dialog
 * @uses jquery/tmpl
 * @uses de_epages/contextualappads/ui/appsdialog
 * @since 7.30.0
 */

/**
 * @cfg {Object} [apps] Array of objects containing the relevant apps. When initializing the dialog, the Array will be filled with the app objects from the parsed JSON object (JSON.parse('#AppsForContext')).
 */

/**
 * @cfg {String} [viewAction] Name of the ViewAction that should be called if the read more button was clicked.
 */

/**
 * @cfg {String} [width] Maximum width of the dialog.
 */

/**
 * See `jQuery.ui.contextualappadsUiAppsdialog` for details.
 *
 * @param {Array}  [apps] Array of objects containing the relevant apps. When initializing the dialog, the Array will be filled with the app objects from the parsed JSON object (JSON.parse('#AppsForContext')).
 * @param {String} [viewAction] Name of the ViewAction that should be called if the read more button was clicked.
 * @param {String} [width] Maximum width of the dialog.
 *
 *
 *
 * ### Examples
 * apps[0] contains an app object with the following properties: (e.g. an app from Bilbee)
 *
 *      appUrl: 'https://www.billbee.de'
 *      description: 'Billbee bietet eine umfangreiche aber einfach zu bedienende...'
 *      id: 'E03F1CBF-A9E0-4D31-99DB-1D9EAFE58F64'
 *      name: 'Billbee'
 *      installed: false
 *      ...
 *
 *
 * @method contextualappadsUiAppsdialog
 * @member jQuery
 *
 * @since 7.30.0
 */

/*
 * @copyright       © Copyright 2006-2018, epages GmbH, All Rights Reserved.
 *
 * @module          de_epages/contextualappads/ui/appsdialog
 */

define("de_epages/contextualappads/ui/appsdialog", [
    "jquery",
    "ep",
    "de_epages",
    "$tmpl!./appsdialog",
    "$dict!ep/dict",

    "jquery/slick/slick",
    "jquery/ui/widget",
    "ep/ui/dialog",
    "jquery/tmpl"
], function($, ep, de_epages, tmplAppDialog, epDict) {
/*
 * @dictionary      ep.dict
 *
 * @translation     {Free}
 *                  {ReadMoreButton}
 */

    // translation object
    var translations = {
            Free: epDict.translate('Free'),
            ReadMoreButton: epDict.translate('ReadMoreButton')
        };

    $.widget('ui.contextualappadsUiAppsdialog', {
    //  dialog,
        tmplElements: {},
        apps: [],

        options: {
            apps: [],
            viewAction: 'MBO-ViewAppDetails',
            width: 960
        },

        /**
         * Method is called during initialization, when widget is created.
         *
         * @member jQuery.ui.contextualappadsUiAppsdialog
         *
         * @since 7.30.0
         */
        _create: function() {
            var self = this,
                o = self.options,
                elem = self.element;

            if (o.apps.length) {
                // filter apps
                // apps are copied by reference, not as a deep copy
                self.apps = o.apps.filter(function(app) {
                    // filter uninstalled apps only
                    return !app.installed;
                });

                // register click event to open the dialog
                elem.on('click.appDialog', function(evt) {
                    self.open();
                });
            }
        },

        /**
         * Method renders the slider template and creates the dialog.
         *
         * @member jQuery.ui.contextualappadsUiAppsdialog
         *
         * @since 7.30.0
         */
        _createDialog: function() {
            var self = this,
                o = self.options,
                elem = self.element,
                slides = (self.apps.length < 3) ? self.apps.length : 3,
                renderedTmpl;

            renderedTmpl = tmplAppDialog([{
                apps: self.apps,
                shopId: o.shopId,
                dict: translations
            }]);

            // Get DOM-Elements from template.
            $.extend(self.tmplElements, renderedTmpl.tmplItem('elements'));
            // Initialize dialog widget. (Width depending on number of apps)
            self.dialog = $('<div/>').html(renderedTmpl).uiDialog({
                width: parseInt(o.width * slides / 3),
                modal: true,
                classes: {
                  'ui-dialog': 'ep-appdialog-dialog'
                },
                position: self._getPosition(elem)
            });

            // initialize the slider plugin
            self._initSlider(slides);

            // Register live-event for ReadMore Buttons inside AppTeaser.
            self.tmplElements.sliderlist.on('click.appReadMore', '.ep-appdialog-slider-teaser-readmore', function() {
                var $this = $(this),
                    appDetails = $this.data('appdetails');

                // first solution => open App in new window (later task see ToDo comment)
                location.href = "?ViewAction=" + o.viewAction + "&appID=" + appDetails.appID + "&ObjectID=" + o.shopId;

                // ToDo: open dialog with further information to install app
                //       will be implemented with UNITY-5123: https://epages.atlassian.net/browse/UNITY-5123
            });

            // adjust position of dialog on resize
            $(window).on('resize.contextualappadsUiAppsdialog', function() {
                if (self.dialog && self.dialog.data('uiUiDialog') !== undefined) {
                    self.dialog.uiDialog('option', {
                        position: self._getPosition(elem)
                    });
                }
            });
        },

        /**
         * Method initializes the slick slider plugin.
         *
         * @member jQuery.ui.contextualappadsUiAppsdialog
         *
         * @since 7.30.0
         */
        _initSlider: function(slides) {
            var self = this;

            self.tmplElements.sliderlist.on('init', function(event, slick) {
                // show/hide arrows (depending on slick variables, see: https://github.com/kenwheeler/slick/)
                self._setArrows(slick);
            });

            // Initialize Slick Slider
            // (Config param: https://github.com/kenwheeler/slick/)
            self.tmplElements.sliderlist.slick({
              infinite: false,
              slidesToShow: slides,
              slidesToScroll: slides,
              prevArrow: '<span class="ep-slick-prev fa fa-chevron-circle-left"></span>',
              nextArrow: '<span class="ep-slick-next fa fa-chevron-circle-right"></span>',
            });

            self.tmplElements.sliderlist.on('afterChange', function(event, slick, currentSlide) {
                // show/hide arrows (depending on slick variables, see: https://github.com/kenwheeler/slick/)
                self._setArrows(slick);
            });

            // recalculate position (to avoid design/position glitches with to many apps filling the complete view port height)
            self.dialog.uiDialog('option', {
                position: self._getPosition(self.element)
            });
        },

        /**
         * Method to set slider arrow elements.
         *
         * @member jQuery.ui.contextualappadsUiAppsdialog
         *
         * @since 7.30.0
         */
         _setArrows: function(slick) {
            // if right border is reached => Hide arrow-element
            slick.$nextArrow[(slick.slideCount - slick.currentSlide <= slick.options.slidesToShow) ? 'hide' : 'show']();
            // if left border is reached => Hide arrow-element
            slick.$prevArrow[(slick.currentSlide === 0) ? 'hide' : 'show']();
        },

        /**
         * Method returns a positioning object.
         *
         * @member jQuery.ui.contextualappadsUiAppsdialog
         *
         * @since 7.30.0
         */
         _getPosition: function(ofElement) {
            return {
                my: "center top",
                at: "center bottom+20",
                of: ofElement
            };
         },

        /**
         * When the method is called it checks if there is already a dialog created or if the dialog is not defined yet. If not defined, the widget function _create() is called otherwise the dialog only opens as an overlay.
         *
         * @method open
         * @member jQuery.ui.contextualappadsUiAppsdialog
         *
         * @since 7.30.0
         */
        open: function() {
            var self = this;

            if (self.dialog) {
                self.dialog.uiDialog('open');
            } else {
                self._createDialog();
            }
        },

        /**
         * When the method is called an open dialog will be closed.
         *
         * @method close
         * @member jQuery.ui.contextualappadsUiAppsdialog
         *
         * @since 7.30.0
         */
        close: function() {
            var self = this;

            if (self.dialog) {
                self.dialog.uiDialog('close');
            }
        },

        destroy: function() {
            var self = this;

            // Deregister live-event for ReadMore Buttons inside AppTeaser.
            self.tmplElements.sliderlist.find('.ep-appdialog-slider-teaser-readmore').off('click.appReadMore')
            // Remove click-event from app button
            self.element.off('click.appDialog');

            // Destroy slider plugin.
            self.tmplElements.sliderlist.slick('unslick');
            // Destroy dialog widget.
            self.dialog.uiDialog('destroy');
            self.dialog = undefined;

            this._superApply(arguments);
        }
    });

    return de_epages;
});


